@@ -1,15 +1,22 @@
 /***********************************************************************
 
-Copyright (c) 2011, 2017, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
-Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
@@ -1135,6 +1142,9 @@ have_conn:
 			}
 		}
 	} else {
+        bool auto_commit = (engine->read_batch_size == 1 &&
+                !(engine->cfg_status & IB_CFG_DISABLE_ROWLOCK)) ? true : false;
+
 		assert(conn_option == CONN_MODE_READ);
 
 		if (!read_crsr) {
@@ -1142,15 +1152,12 @@ have_conn:
 				/* This is read operation, start a trx
 				with "read_write" parameter set to false */
 				conn_data->crsr_trx = ib_cb_trx_begin(
-					engine->trx_level, false,
-					engine->read_batch_size == 1);
+					engine->trx_level, false, auto_commit);
 				trx_updated = true;
 			} else {
 				ib_cb_trx_start(conn_data->crsr_trx,
 						engine->trx_level,
-						false,
-						engine->read_batch_size == 1,
-						NULL);
+						false, auto_commit, NULL);
 			}
 
 			err = innodb_api_begin(
@@ -1183,8 +1190,7 @@ have_conn:
 			/* This is read operation, start a trx
 			with "read_write" parameter set to false */
 			conn_data->crsr_trx = ib_cb_trx_begin(
-				engine->trx_level, false,
-				engine->read_batch_size == 1);
+				engine->trx_level, false, auto_commit);
 
 			trx_updated = true;
 
@@ -1230,8 +1236,7 @@ have_conn:
 			with "read_write" parameter set to false */
 			ib_cb_trx_start(conn_data->crsr_trx,
 					engine->trx_level,
-					false,
-					engine->read_batch_size == 1,
+					false, auto_commit,
 					NULL);
 
 			ib_cb_cursor_stmt_begin(conn_data->read_crsr);
@@ -1691,6 +1696,8 @@ convert_to_char(
 			int8_t		int_val = *(int8_t*)value;
 			snprintf(buf, buf_len, "%" PRIi8, int_val);
 		}
+	} else {
+		return 0;
 	}
 
 	return(strlen(buf));
@@ -1844,6 +1851,8 @@ search_done:
 
 		result->col_value[MCI_COL_VALUE].value_str = conn_data->row_buf;
 		result->col_value[MCI_COL_VALUE].value_len = strlen(table_name);
+		result->col_value[MCI_COL_VALUE].is_str = true;
+		result->col_value[MCI_COL_VALUE].is_valid = true;
 	}
 
 	result->col_value[MCI_COL_KEY].value_str = (char*)key;
@@ -1956,6 +1965,8 @@ search_done:
 
 		result->col_value[MCI_COL_VALUE].value_str = conn_data->mul_col_buf;
 		result->col_value[MCI_COL_VALUE].value_len = total_len;
+		result->col_value[MCI_COL_VALUE].is_str = true;
+		result->col_value[MCI_COL_VALUE].is_valid = true;
 		((char*)result->col_value[MCI_COL_VALUE].value_str)[total_len] = 0;
 
 		free(result->extra_col_value);
@@ -1964,6 +1975,7 @@ search_done:
 		unsigned int	int_len;
 		char		int_buf[MAX_INT_CHAR_LEN];
 
+		memset(int_buf, 0, sizeof int_buf);
 		int_len = convert_to_char(
 			int_buf, sizeof int_buf,
 			&result->col_value[MCI_COL_VALUE].value_int,
@@ -1985,6 +1997,8 @@ search_done:
 			 conn_data->mul_col_buf;
 
 		result->col_value[MCI_COL_VALUE].value_len = int_len;
+		result->col_value[MCI_COL_VALUE].is_str = true;
+		result->col_value[MCI_COL_VALUE].is_valid = true;
 	}
 
         *item = result;
