@@ -1,13 +1,20 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -56,14 +63,6 @@
 using std::min;
 using std::max;
 
-/* max size of log messages (error log, plugins' logging, general log) */
-#ifdef WITH_WSREP
-/* make this bigger to accomodate wsrep_provider_options */
-static const uint MAX_LOG_BUFFER_SIZE= 2048;
-#else
-static const uint MAX_LOG_BUFFER_SIZE= 1024;
-#endif /* WITH_WSREP */
-
 #ifndef _WIN32
 static int   log_syslog_facility= 0;
 #endif
@@ -1992,10 +1991,10 @@ void log_slow_do(THD *thd)
   THD_STAGE_INFO(thd, stage_logging_slow_query);
   thd->status_var.long_query_count++;
 
-  if (thd->rewritten_query.length())
+  if (thd->rewritten_query().length())
     query_logger.slow_log_write(thd,
-                                thd->rewritten_query.c_ptr_safe(),
-                                thd->rewritten_query.length());
+                                thd->rewritten_query().ptr(),
+                                thd->rewritten_query().length());
   else
     query_logger.slow_log_write(thd, thd->query().str, thd->query().length);
 }
