@@ -435,6 +435,11 @@ static inline int my_thread_set_THR_THD(THD *thd)
   return my_set_thread_local(THR_THD, thd);
 }
 
+/**
+  Set m_opt_tracking_mode with a user given value associated with sysvar.
+*/
+void set_mysqld_opt_tracking_mode();
+
 #ifdef HAVE_PSI_INTERFACE
 
 C_MODE_START
@@ -1005,6 +1010,7 @@ enum options_mysqld
   OPT_SSL_KEY,
   OPT_UPDATE_LOG,
   OPT_WANT_CORE,
+  OPT_COREDUMPER,
   OPT_LOG_ERROR,
 #ifdef WITH_WSREP
   OPT_WSREP_PROVIDER,
@@ -1132,6 +1138,11 @@ static inline THD *_current_thd(void)
 bool update_named_pipe_full_access_group(const char *new_group_name);
 #endif
 
+/* coredumper */
+extern bool  opt_libcoredumper;
+extern char *opt_libcoredumper_path;
+bool         validate_libcoredumper_path(char *opt_libcoredumper_path);
+
 #ifdef WITH_WSREP
 extern "C" void *start_wsrep_THD(void*);
 typedef void (*wsrep_thd_processor_fun)(THD *);
