@@ -31,8 +31,78 @@
 #include <cstdlib>
 #include "log_event.h"
 #include <rpl_slave.h>
-#include "sql_base.h"           // TEMP_PREFIX 
+#include "sql_base.h"           // TEMP_PREFIX
 #include "rpl_msr.h"           // channel_map
+#include <functional>
+#include <sstream>
+#include <string>
+
+/* We don't need PSI instrumentation for regex mutex, however
+   Regex class has its constructor compiled conditionally.
+   Disable instrumentation.
+   Regex class uses c++11 nullptr. As we are in c++03 world
+   let's define nullptr to satisfy compiler */
+#ifdef HAVE_PSI_INTERFACE
+#undef HAVE_PSI_INTERFACE
+#endif
+#define nullptr 0
+#include <sql_regex.h>
+
+/* Check if the string is valid. In method name we allow only:
+   alpha-num
+   underscore
+   dash
+   dot
+   but we do not allow leading and trailing dash/dot.
+   Method names have to be separated by colons (spaces before/after colon are allowed).
+   Below regex may seem difficult, but in fact it is not:
+
+   1. Any number of leading spaces
+
+     2. Followed by  alpha-num character or underscore
+       3. Followed by any number of underscore/dash/dot
+       4. Followed by alpha-num character or underscore
+       5. 3 - 4 group is optional
+     6. Followed by colon (possibly with leading and/or trailing spaces)
+     7. 2 - 6 group is optional
+
+   8. Followed by alpha-num character or underscore
+     9. Followed by any number of underscore/dash/dot
+     10. Followed by alpha-num character or underscore
+     11. 9 -10 group is optional
+
+   11. Followed by any number of trailing spaces
+
+   It is the same as:
+   ^\s*(\w([-_.]*\w)*(\s*,\s*))*\w([-_.]*\w)*\s*$    */
+
+#define COLON_REGEX "([[:blank:]]*,[[:blank:]]*)"
+#define METHOD_REGEX "[[:alnum:]_]([-_.]*[[:alnum:]_])*"
+#define SST_ALLOWED_METHODS_REGEX_PATTERN \
+    "^[[:blank:]]*(" METHOD_REGEX COLON_REGEX ")*" METHOD_REGEX "[[:blank:]]*$"
+
+static Regex sst_allowed_methods_regex;
+
+/* We allow custom sst scripts, so data can be anything.
+
+   We could create and maintain the list of forbidden
+   characters and the ways they could be used to inject the command to
+   the OS. However this approach seems to be too error prone.
+   Instead of this we will just allow alpha-num + a few special characters
+   (colon, slash, dot, underscore, square brackets, hyphen).
+
+   It is the same as ^[\w:/.[\]-]+$    */
+
+   /* For some reason regex engine is not able to handle the following pattern
+   allowing square brackets:
+   "^[[:alnum:]:/._[\\]-]+$"
+   We will substitute square brackets with lt/gt for validation.
+   Because of another regex engine problem, hyphen literal has to be located
+   at the end of matching set (cannot be escaped when in the middle). */
+static const char *sst_method_allowed_chars_regex_pattern=
+    "^[[:alnum:]:/._<>-]+$";
+static Regex sst_method_allowed_chars_regex;
+
 
 extern const char wsrep_defaults_file[];
 extern const char wsrep_defaults_group_suffix[];
@@ -73,6 +143,15 @@ const char* wsrep_sst_receive_address = WSREP_SST_ADDRESS_AUTO;
 const char* wsrep_sst_donor           = "";
       char* wsrep_sst_auth            = NULL;
 
+#define WSREP_SST_ALLOWED_METHODS_DEFAULT WSREP_SST_MYSQLDUMP "," \
+                                          WSREP_SST_RSYNC "," \
+                                          WSREP_SST_SKIP "," \
+                                          WSREP_SST_XTRABACKUP "," \
+                                          WSREP_SST_XTRABACKUP_V2
+
+const char *wsrep_sst_allowed_methods = WSREP_SST_ALLOWED_METHODS_DEFAULT;
+static std::vector<std::string> allowed_sst_methods;
+
 // container for real auth string
 static const char* sst_auth_real      = NULL;
 my_bool wsrep_sst_donor_rejects_queries = FALSE;
@@ -234,6 +313,65 @@ bool wsrep_before_SE()
           && strcmp (wsrep_sst_method, WSREP_SST_MYSQLDUMP));
 }
 
+/* Helper class for splitting string tokens separated with
+   commas. Spaces are stripped. */
+struct Splitter : public std::unary_function<char, void> {
+  Splitter(std::vector<std::string> &vec) : vec_(vec) {}
+
+  void flush()
+  {
+    if (tmp_.length() > 0)
+    {
+      vec_.push_back(tmp_);
+      tmp_.clear();
+    }
+  }
+  void operator()(char ch)
+  {
+    if (ch == ',')
+    {
+      vec_.push_back(tmp_);
+      tmp_.clear();
+    }
+    else
+    {
+      if (ch != ' ')
+      {
+        tmp_+= ch;
+      }
+    }
+  }
+  std::vector<std::string> &vec_;
+  std::string               tmp_;
+};
+
+bool wsrep_setup_allowed_sst_methods()
+{
+  sst_allowed_methods_regex.compile(SST_ALLOWED_METHODS_REGEX_PATTERN,
+                                    MY_REG_EXTENDED | MY_REG_NOSUB,
+                                    &my_charset_latin1);
+  sst_method_allowed_chars_regex.compile(
+      sst_method_allowed_chars_regex_pattern, MY_REG_EXTENDED | MY_REG_NOSUB,
+      &my_charset_latin1);
+
+  std::string methods(wsrep_sst_allowed_methods);
+
+  if (!sst_allowed_methods_regex.match(methods))
+  {
+    WSREP_ERROR(
+        "Wrong format of --wsrep-sst-allowed-methods parameter value: %s",
+        wsrep_sst_allowed_methods);
+    return true;
+  }
+
+  // split it into tokens and strip
+  Splitter splitter(allowed_sst_methods);
+  std::for_each(methods.begin(), methods.end(), splitter).flush();
+
+  return false;
+}
+
+
 static bool            sst_complete = false;
 static bool            sst_needed   = false;
 
@@ -554,10 +692,8 @@ static void* sst_joiner_thread (void* a)
       }
       else
       {
-        // Clear the pointer to SST process:
-        if (mysql_mutex_lock (&LOCK_wsrep_sst)) abort();
-        sst_process = NULL;
-        mysql_mutex_unlock (&LOCK_wsrep_sst);
+        // Do not clear the pointer to SST process. It will be useful if 
+        // we decide to interrupt it.
         err = 0;
       }
     }
@@ -584,10 +720,16 @@ static void* sst_joiner_thread (void* a)
     mysql_cond_signal  (&arg->COND_wsrep_sst_thread);
     mysql_mutex_unlock (&arg->LOCK_wsrep_sst_thread); //! @note arg is unusable after that.
 
-    if (err) return NULL; /* lp:808417 - return immediately, don't signal
-                           * initializer thread to ensure single thread of
-                           * shutdown. */
+    if (err)
+    {
+      if (mysql_mutex_lock (&LOCK_wsrep_sst)) abort();
+      sst_process = NULL;
+      mysql_mutex_unlock (&LOCK_wsrep_sst);
 
+      return NULL; /* lp:808417 - return immediately, don't signal
+                    * initializer thread to ensure single thread of
+                    * shutdown. */
+    }
     wsrep_uuid_t  ret_uuid  = WSREP_UUID_UNDEFINED;
     wsrep_seqno_t ret_seqno = WSREP_SEQNO_UNDEFINED;
 
@@ -627,6 +769,10 @@ static void* sst_joiner_thread (void* a)
   wsrep_pfs_delete_thread();
 #endif /* HAVE_PSI_INTERFACE */
 
+  if (mysql_mutex_lock (&LOCK_wsrep_sst)) abort();
+  sst_process = NULL;
+  mysql_mutex_unlock (&LOCK_wsrep_sst);
+
   return NULL;
 }
 
@@ -781,7 +927,7 @@ static ssize_t sst_prepare_mysqldump (const char*  addr_in,
   return ret;
 }
 
-static bool SE_initialized = false;
+static enum Wsrep_SE_init_result SE_initialized= WSREP_SE_INIT_RESULT_NONE;
 
 ssize_t wsrep_sst_prepare (void** msg, THD* thd)
 {
@@ -1311,7 +1457,7 @@ static int sst_donate_other (const char*   method,
   {
     WSREP_ERROR("sst_donate_other(): pthread_create() failed: %d (%s)",
                 ret, strerror(ret));
-    return ret;
+    return -ret;
   }
   mysql_cond_wait (&arg.COND_wsrep_sst_thread, &arg.LOCK_wsrep_sst_thread);
 
@@ -1320,16 +1466,94 @@ static int sst_donate_other (const char*   method,
   return arg.err;
 }
 
+/*
+  Validate SST request string.
+  The protocol is: method\0data\0
+
+  For xtrabackup-v2 it is: method\0addresspath\0
+  like:
+  "xtrabackup-v2\000127.0.0.1:4444/xtrabackup_sst//1"
+*/
+static bool is_sst_request_valid(const std::string &msg)
+{
+  size_t method_len= strlen(msg.c_str());
+
+  if (method_len == 0)
+  {
+    return false;
+  }
+
+  std::string method= msg.substr(0, method_len);
+
+  // Is this method allowed?
+  std::vector<std::string>::iterator res= std::find(
+      allowed_sst_methods.begin(), allowed_sst_methods.end(), method);
+  if (res == allowed_sst_methods.end())
+  {
+    return false;
+  }
+
+  const char *data_ptr= msg.c_str() + method_len + 1;
+  size_t      data_len= strlen(data_ptr);
+
+  // method + null + data + null
+  if (method_len + 1 + data_len + 1 != msg.length())
+  {
+    // Someone tries to piggyback after 2nd null
+    return false;
+  }
+
+  if (data_len > 0)
+  {
+    std::string data= msg.substr(method_len + 1, data_len);
+
+    // first make sure we don't have lt/gt which are not allowed
+    if (std::string::npos != data.find('<') ||
+        std::string::npos != data.find('>'))
+    {
+        return false;
+    }
+
+    // square brackets workaround
+    std::replace(data.begin(), data.end(), '[', '<');
+    std::replace(data.begin(), data.end(), ']', '>');
+
+    return sst_method_allowed_chars_regex.match(data);
+  }
+  return true;
+}
+
+/* Helper functor class for conversion of std::string
+   containing null characters into printable string */
+struct StringBuilder : public std::unary_function<char, void> {
+  StringBuilder(std::ostringstream &ss) : ss_(ss) {}
+
+  void operator()(char ch)
+  {
+    if (ch != 0)
+      ss_ << ch;
+    else
+      ss_ << "<nullptr>";
+  }
+  std::ostringstream &ss_;
+};
+
 wsrep_cb_status_t wsrep_sst_donate_cb (void* app_ctx, void* recv_ctx,
                                        const void* msg, size_t msg_len,
                                        const wsrep_gtid_t* current_gtid,
                                        const char* state, size_t state_len,
                                        bool bypass)
 {
-  /* This will be reset when sync callback is called.
-   * Should we set wsrep_ready to FALSE here too? */
-//  wsrep_notify_status(WSREP_MEMBER_DONOR);
-  local_status.set(WSREP_MEMBER_DONOR);
+
+  std::string message(reinterpret_cast<const char *>(msg), msg_len);
+  if (!is_sst_request_valid(message))
+  {
+    std::ostringstream ss;
+    StringBuilder      stringBuilder(ss);
+    std::for_each(message.begin(), message.end(), stringBuilder);
+    WSREP_ERROR("Invalid sst_request: %s", ss.str().c_str());
+    return WSREP_CB_FAILURE;
+  }
 
   const char* method = (char*)msg;
   size_t method_len  = strlen (method);
@@ -1338,6 +1562,16 @@ wsrep_cb_status_t wsrep_sst_donate_cb (void* app_ctx, void* recv_ctx,
   char uuid_str[37];
   wsrep_uuid_print (&current_gtid->uuid, uuid_str, sizeof(uuid_str));
 
+  DBUG_EXECUTE_IF("wsrep_sst_donate_cb_fails", 
+  {
+    return WSREP_CB_FAILURE;
+  });
+
+  /* This will be reset when sync callback is called.
+   * Should we set wsrep_ready to FALSE here too? */
+//  wsrep_notify_status(WSREP_MEMBER_DONOR);
+  local_status.set(WSREP_MEMBER_DONOR);
+
   wsp::env env(NULL);
   if (env.error())
   {
@@ -1377,17 +1611,18 @@ wsrep_cb_status_t wsrep_sst_donate_cb (void* app_ctx, void* recv_ctx,
                            current_gtid->seqno, bypass, env());
   }
 
+  /* Above methods should return 0 in case of success and negative value
+   * in case of failure. If we have any positive value here it means that we
+   * handle errors in above functions in the wrong way */
+  DBUG_ASSERT(ret <= 0); 
   return (ret >= 0 ? WSREP_CB_SUCCESS : WSREP_CB_FAILURE);
 }
 
-void wsrep_SE_init_grab()
+enum Wsrep_SE_init_result wsrep_SE_init_wait(THD* thd)
 {
-  if (mysql_mutex_lock (&LOCK_wsrep_sst_init)) abort();
-}
-
-void wsrep_SE_init_wait(THD* thd)
-{
-  while (SE_initialized == false && thd->killed == THD::NOT_KILLED)
+  mysql_mutex_lock (&LOCK_wsrep_sst_init);
+  while (SE_initialized == WSREP_SE_INIT_RESULT_NONE &&
+         thd->killed == THD::NOT_KILLED)
   {
     mysql_mutex_lock(&thd->LOCK_thd_data);
     thd->current_cond= &COND_wsrep_sst_init;
@@ -1406,26 +1641,28 @@ void wsrep_SE_init_wait(THD* thd)
     thd->current_mutex= NULL;
     mysql_mutex_unlock(&thd->LOCK_thd_data);
   }
+  enum Wsrep_SE_init_result ret= SE_initialized;
   mysql_mutex_unlock (&LOCK_wsrep_sst_init);
 
   mysql_mutex_lock(&thd->LOCK_thd_data);
   thd->current_cond= NULL;
   thd->current_mutex= NULL;
   mysql_mutex_unlock(&thd->LOCK_thd_data);
+  return ret;
 }
 
-void wsrep_SE_init_done()
+void wsrep_SE_initialized(enum Wsrep_SE_init_result result)
 {
+  mysql_mutex_lock (&LOCK_wsrep_sst_init);
+  if (SE_initialized == WSREP_SE_INIT_RESULT_NONE)
+  {
+    SE_initialized= result;
+  }
   mysql_cond_signal (&COND_wsrep_sst_init);
   mysql_mutex_unlock (&LOCK_wsrep_sst_init);
 }
 
-void wsrep_SE_initialized()
-{
-  SE_initialized = true;
-}
-
 bool wsrep_is_SE_initialized()
 {
-  return SE_initialized;
+  return SE_initialized == WSREP_SE_INIT_RESULT_SUCCESS;
 }
