@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2020, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -706,6 +706,7 @@ public:
 	void iterate(Callback& callback) const
 		UNIV_NOTHROW
 	{
+		m_mutex.enter();
 		Counters::const_iterator	end = m_counters.end();
 
 		for (Counters::const_iterator it = m_counters.begin();
@@ -714,6 +715,7 @@ public:
 
 			callback(*it);
 		}
+		m_mutex.exit();
 	}
 
 	/** Disable the monitoring */
@@ -773,7 +775,7 @@ private:
 	typedef std::vector<Count*> Counters;
 
 	/** Mutex protecting m_counters */
-	Mutex			m_mutex;
+	mutable Mutex		m_mutex;
 
 	/** Counters for the latches */
 	Counters		m_counters;
