@@ -3,13 +3,21 @@
 Copyright (c) 2014, 2015, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2016, Percona Inc. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -176,7 +184,6 @@ extern PSI_memory_key	mem_key_log_online_sys;
 extern PSI_memory_key	mem_key_log_online_read_buf;
 extern PSI_memory_key	mem_key_log_online_iterator_files;
 extern PSI_memory_key	mem_key_log_online_iterator_page;
-extern PSI_memory_key	mem_key_trx_distinct_page_access_hash;
 extern PSI_memory_key	mem_key_parallel_doublewrite;
 
 /** Setup the internal objects needed for UT_NEW() to operate.
