@@ -420,8 +420,10 @@ wsrep_row_upd_check_foreign_constraints(
 		NOT break the constraint. */
 
 		if (foreign->foreign_index == index
-		    && row_upd_changes_first_fields_binary(
-		        entry, index, node->update, foreign->n_fields)) {
+		    && (node->is_delete
+			|| row_upd_changes_first_fields_binary(
+				entry, index, node->update,
+				foreign->n_fields))) {
 
 			if (foreign->referenced_table == NULL) {
 				foreign->referenced_table = 
