@@ -32,6 +32,9 @@
 #include "rocksdb/slice.h"
 #include "rocksdb/status.h"
 
+/* MyRocks header files */
+#include "./rdb_global.h"
+
 #ifdef HAVE_JEMALLOC
 #include <jemalloc/jemalloc.h>
 #endif
